import React, { useContext } from 'react';
import { useLocation, matchPath } from 'react-router-dom';
import { GamesContext } from '../../context/games';

import { Color, Player, Board } from '../../components/Checkers';
import './GameBoard.css';

export default function GameBoard({ username, onStoneClick, onStoneMove }) {
  const game = useSelectedGame();

  const opponentColor = Color.opposite(game?.myColor);
  const [opponentName, myName] = (game?.opponentName) ? [game.opponentName, username] : ['', ''];
  const flipBoard = (game?.myColor === Color.black) ? true : null;

  const optionalOnStoneClick = (typeof onStoneClick !== 'function') ? null :
    (cellId) => onStoneClick(game?.uuid, cellId);

  const optionalOnStoneMove = (typeof onStoneMove !== 'function') ? null :
    (move) => onStoneMove(game?.uuid, move);

  return (
    <div className='GameBoard'>
      <Player color={opponentColor || Color.black} name={opponentName} />
      <Board board={game?.board} flip={flipBoard}
        onStoneClick={optionalOnStoneClick}
        onStoneMove={optionalOnStoneMove}
      />
      <Player color={game?.myColor || Color.white} name={myName} />
      {game?.isPushingGameMove ? <span>Moving...</span> : null /* TODO: isPushing shall be stored per game. curernty it is global indicator */} 
    </div>
  )
}

export function useSelectedGame() {
  const games = useContext(GamesContext);
  const { pathname } = useLocation();

  if (matchPath('/games/new', pathname))
    return games.newGame;

  if (matchPath('/games/proposal', pathname))
    return games.findGame({ uuid: games.proposal.selectedUUID });

  if (matchPath('/games/active', pathname))
    return games.findGame({ uuid: games.active.selectedUUID });

  if (matchPath('/games/archive', pathname))
    return games.findGame({ uuid: games.archive.selectedUUID });

  return undefined;
}