import { useReducer } from 'react';
import { nextState } from '../util/StateHelper';
import { defaultBoard } from '../components/Checkers';

export const gamesInitialState = {
  gamesList: null,

  newGame: {
    opponentName: '',
    myColor: '',
    board: defaultBoard,
    message: '',
  },

  proposal: {
    selectedUUID: null,
    message: '',
  },

  active: {
    selectedUUID: null,
    message: '',
  },

  archive: {
    selectedUUID: null,
  },

  // Network
  isPollingGamesList: false,
  isPushingNewGame: false,

  isPushingGameProposalCancel: false,
  isPushingGameProposalReject: false,
  isPushingGameProposalAccept: false,

  isPushingGameSurrender: false,
  isPushingGameDrawRequest: false,
  isPushingGameDrawAccept: false,
  isPushingGameDrawReject: false,
  isPushingGameMove: false,

  findGame,
  nextGame,
};

function reducer(state, action) {
  switch (action.type) {

    case 'next':
      return nextState(state, action);

    case 'nextNewGame':
      return {
        ...state,
        newGame: nextState(state.newGame, action)
      };

    case 'nextProposal':
      return {
        ...state,
        proposal: nextState(state.proposal, action)
      };

    case 'nextActive':
      return {
        ...state,
        active: nextState(state.active, action)
      };

    case 'nextArchive':
      return {
        ...state,
        archive: nextState(state.archive, action)
      };

    default:
      throw Error('GamesReducer: unknown action.type', action.type);
  }
}

export default function useGamesReducer() {
  return useReducer(reducer, gamesInitialState);
}

function findGame({ uuid }) {
  return this.gamesList?.find((game) => game.uuid === uuid);
}

function nextGame(nextGame) {
  return this.gamesList?.map((game) => (game.uuid === nextGame?.uuid) ? nextGame : game);
}